#!/usr/bin/perl -w

use Test::More 'no_plan';

package Catch;

sub TIEHANDLE {
    my($class, $var) = @_;
    return bless { var => $var }, $class;
}

sub PRINT  {
    my($self) = shift;
    ${'main::'.$self->{var}} .= join '', @_;
}

sub OPEN  {}    # XXX Hackery in case the user redirects
sub CLOSE {}    # XXX STDERR/STDOUT.  This is not the behavior we want.

sub READ {}
sub READLINE {}
sub GETC {}
sub BINMODE {}

my $Original_File = 'root/etc/e-smith/web/panels/manager/cgi-bin/localnetworks';

package main;

# pre-5.8.0's warns aren't caught by a tied STDERR.
$SIG{__WARN__} = sub { $main::_STDERR_ .= join '', @_; };
tie *STDOUT, 'Catch', '_STDOUT_' or die $!;
tie *STDERR, 'Catch', '_STDERR_' or die $!;

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 48 root/etc/e-smith/web/panels/manager/cgi-bin/localnetworks

use esmith::FormMagick::Tester;
use esmith::TestUtils;
use esmith::TestUtils;
use esmith::ConfigDB;
use esmith::AccountsDB;

my $panel = $Original_File;
my $ua = esmith::FormMagick::Tester->new();

my $c = esmith::ConfigDB->open();
my $a = esmith::AccountsDB->open();

is  (mode($panel), '4750',              "Check permissions on script");
ok  ($ua->get_panel($panel),            "ABOUT TO RUN L10N TESTS");
is  ($ua->{status}, 200,                "200 OK");
#like($ua->{content}, qr/FORM_TITLE/,   "Saw untranslated form title");
ok  ($ua->set_language("en-us"),        "Set language to U.S. English");
ok  ($ua->get_panel($panel),            "Get panel");
is  ($ua->{status}, 200,                "200 OK");
like($ua->{content}, qr/Local networks/, "Saw translated form title");


# 
# Testing creating local networks
#

ok  ($ua->get_panel($panel),            "ABOUT TO TEST CREATING NETWORK");
ok  ($ua->follow("Click here"),         "Follow 'create network' link");
is  ($ua->{status}, 200,                "200 OK");
like($ua->{content}, qr/Add a local network/,   "Saw page title");
like($ua->{content}, qr/Network address/,       "Saw network address field");
like($ua->{content}, qr/Subnet mask/,           "Saw subnet mask field");
like($ua->{content}, qr/Router/,                "Saw router field");
like($ua->{content}, qr/Add/,                   "Saw add button");

#
# checking error handling
#

$ua->field(networkAddress => '10.0.0.10000');
$ua->field(networkMask    => '10.0.0.10000');
$ua->click("Add");
# like($ua->{content}, qr/Invalid network address/,  "Saw network address validation message");
like($ua->{content}, qr/Invalid subnet mask/,      "Saw submask validation message");

#
# now let's actually add one
#

SKIP: {

    skip 10, "Unsafe!" unless destruction_ok();

    ok  ($ua->follow("Click here"),         "ACTUALLY ADDING A NETWORK");
    $ua->field(networkAddress => '10.0.0.0');
    $ua->field(networkMask    => '255.255.255.0');
    $ua->field(networkRouter  => '10.0.0.1');
    $ua->click("Add");
    like($ua->{content}, qr/Successfully added network/,  "Saw success message");

    ok  ($ua->follow("Click here"),         "ADDING NETWORK WITH DEFAULT ROUTER");
    $ua->field(networkAddress => '10.0.0.0');
    $ua->field(networkMask    => '255.255.255.0');

    ok  ($ua->follow("Remove"),         "REMOVING NETWORK");
    like($ua->{content}, qr/Are you sure/,  "Saw confirmation message");
    $ua->click("Remove");
    like($ua->{content}, qr/Successfully deleted/,  "Saw success message");

}


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

